@@ -66,7 +66,7 @@ function start-crawl {
     --debug)
       verbose=1
       ;;
-    --no-purge|--summary)
+    --no-purge|--summary|--print-files)
       ;;
     --errors|--files|--found|--not-found|--directory)
       pos=$[${pos}+1]
@@ -125,6 +125,9 @@ function start-crawl {
     --summary)
       grep -E "^HTTrack Website Copier/[^ ]* mirror complete in " "${tmp}/hts-log.txt"
       ;;
+    --print-files)
+      find "${tmp}" -mindepth 1 -type f
+      ;;
     --errors)
       shift
       assert_equals "checking errors" "$1" "$(grep -iEc "^[0-9\:]*[[:space:]]Error:" "${tmp}/hts-log.txt")"
